/**
  *This is the primary graphical object for our Graph Mapper program.
  */
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JColorChooser;
import javax.swing.JScrollPane;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import javax.swing.KeyStroke;
import javax.swing.event.MenuListener;
import javax.swing.event.MenuEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.Color;
//needed for "save settings"
import java.io.FileReader;
import java.io.FileWriter;
import java.io.File;
import java.io.BufferedWriter;
import java.io.BufferedReader;

public class GraphGUI extends JFrame implements ActionListener, Runnable, MenuListener
{

   //create objects for menubar
   private JMenuBar mb_top = new JMenuBar();
     private JMenu  m_file = new JMenu("File");
       private JMenu m_new  = new JMenu("New");
         private JMenuItem mi_new_d = new JMenuItem("Directed/Weighted",KeyEvent.VK_W);
         private JMenuItem mi_new_1 = new JMenuItem("Directed/Unweighted",KeyEvent.VK_U);
         private JMenuItem mi_new_2 = new JMenuItem("Undirected/Weighted",KeyEvent.VK_E);
         private JMenuItem mi_new_u = new JMenuItem("Undirected/Unweighted",KeyEvent.VK_N);
       private JMenuItem mi_sav = new JMenuItem("Save",KeyEvent.VK_S);
       private JMenuItem mi_sava= new JMenuItem("Save As...",KeyEvent.VK_A);
       private JMenuItem mi_load = new JMenuItem("Load",KeyEvent.VK_L);
       private JMenuItem mi_exit = new JMenuItem("Exit",KeyEvent.VK_X);
     private JMenu  m_options = new JMenu("Options");
       private JMenu m_color = new JMenu("Colors");
         private JMenuItem mi_bcolor = new JMenuItem("Background",KeyEvent.VK_G);
         private JMenuItem mi_ncolor = new JMenuItem("Nodes",KeyEvent.VK_O);
         private JMenuItem mi_lcolor = new JMenuItem("Lines",KeyEvent.VK_L);
         private JMenuItem mi_tcolor = new JMenuItem("Text",KeyEvent.VK_T);
       private JMenuItem mi_radius = new JMenuItem("Set Node Radius",KeyEvent.VK_R);
       private JMenuItem mi_original= new JMenuItem("Restore Defaults",KeyEvent.VK_D);
       private JMenuItem mi_custom  = new JMenuItem("Remove Customization",KeyEvent.VK_C);
       private JMenuItem mi_save = new JMenuItem("Save Settings",KeyEvent.VK_S);
     private JMenu m_view = new JMenu("View");
       private JMenuItem mi_clear = new JMenuItem("Clear Search",KeyEvent.VK_S);
       private JCheckBoxMenuItem mi_weight = new JCheckBoxMenuItem("Display Weights");
       private JCheckBoxMenuItem mi_direction = new JCheckBoxMenuItem("Display Direction");
     private JMenu m_help = new JMenu("Help");
       private JMenuItem mi_about = new JMenuItem("About",KeyEvent.VK_A);

   /**
     *number of instances currently open.
     */
   protected static int instances = 0;

   //creates our JGraph
   private JGraph graph;

   //creates a JScrollPane with JGrpah in it
   private JScrollPane jsp_graph = new JScrollPane(graph);

   /**
     *title of our window
     */
   private String title = new String();


   /**
     *constructor used to create either a weighted or unweighted graph GUI.
     *@param b true if weighted/directed, false if unweighted/undirected
     */
   public GraphGUI(boolean b)
   {
      graph = new JGraph(b); //give it a new directed or undirected graph

      //checks for default INI file and loads it if it exists
      //we cant't do this in construct() becasue it will overwrite a command line loaded file
      File ini = new File("./.gmapper.ini");
      if(ini.exists())
        {
           INI.load(graph, ini);
           graph.setModified(false); //undoes the modifications of INI
        }

      start(); //start our scrollbar thread
      SearchPanel sb = new SearchPanel(graph, this);//starts searchpanel thread
      construct(); //establish our gui
   }


   public GraphGUI(boolean weighted, boolean directed)
   {
      graph = new JGraph(weighted, directed); //give it a new directed or undirected graph

      //checks for default INI file and loads it if it exists
      //we cant't do this in construct() becasue it will overwrite a command line loaded file
      File ini = new File("./.gmapper.ini");
      if(ini.exists())
        {
           INI.load(graph, ini);
           graph.setModified(false); //undoes the modifications of INI
        }

      start(); //start our scrollbar thread
      SearchPanel sb = new SearchPanel(graph, this);//starts searchpanel thread
      construct(); //establish our gui
   }

   /**
     *constructor for creating an empty graph GUI or one based of a file.
     *If file is set to null, new Graph is assumed and user is prompted
     *@param file the file to open or null for a new document.
     */
   public GraphGUI(File file)
   {
      //checks to see if user wants a directed or undirected graph if this is the first one the user is creating
      if (file == null)
      {
         String[] options = {"Directed/Weighted","Directed/Unweighted","Undirected/Weighted","Undirected/Unweighted",};
         String type = (String) JOptionPane.showInputDialog(new JFrame(),"What type of graph would you like?","Graph Type",JOptionPane.QUESTION_MESSAGE,null,options,options[0]);
         if(type == options[3])
           { graph = new JGraph(false); }
         else if(type == options[0])
           { graph = new JGraph(); }
         else if(type == null)
           { //assume they didn't want a graph to being with :)
              System.exit(2);
           }
         else if(type == options[1])
           {
              graph = new JGraph(false,true);
           }
         else if(type == options[2])
           {
              graph = new JGraph(true,false);
           }
         else //to prevent null pointer
           { graph = new JGraph(); }

         
        //checks for default INI file and loads it if it exists -- BEFORE WE LOAD GRAPH so not to overwrite
        //in if statement becasue it only needs to run if graph is instantiated 
        File ini = new File("./.gmapper.ini");
        if(ini.exists())
        {
           INI.load(graph, ini);
           graph.setModified(false); //undoes the modifications of INI
        }

      }



      else if( file != null) //command line arg for file
      {
         graph = new JGraph();
         graph.load(file);
      }
      else
      { graph = new JGraph(); } //avoid null pointer exceptions

      //start scrollbar updater
      start();
      SearchPanel sb = new SearchPanel(graph,this); //starts searchpanel thread
      construct(); //establish GUI
   }

   //You know what I hate? The fact that a call to this(); must be the first call in the function. I would rather
   //establish a few values first in case they're non default. In future program I think I'm going to adopt the convention
   //of an actual construct() method to handle the graphical components so I can establish some other values first if needed
   /**
     *contructs graphical components.
     *desigend to be run after constructor sets defaults for GUI
     */
   private void construct()
   {
      //sets some stuff for our window
      getContentPane().setLayout(new BorderLayout()); //the layout
      setTitle("Graph Mapper"); //set the window title

      //construct the menubar
      mb_top.add(m_file);
       m_file.setMnemonic(KeyEvent.VK_F);
       m_file.add(m_new);
         m_new.setMnemonic(KeyEvent.VK_N);
         m_new.add(mi_new_d);
           mi_new_d.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.CTRL_MASK));
         m_new.add(mi_new_1);
           mi_new_1.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, ActionEvent.CTRL_MASK));
         m_new.add(mi_new_2);
           mi_new_2.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, ActionEvent.CTRL_MASK));
         m_new.add(mi_new_u);
           mi_new_u.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_4, ActionEvent.CTRL_MASK));
       m_file.add(mi_load);
         mi_load.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
       m_file.add(mi_sav);
         mi_sav.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
       m_file.add(mi_sava);
         mi_sava.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
       m_file.addSeparator();
       m_file.add(mi_exit);
         mi_exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,ActionEvent.CTRL_MASK));
      mb_top.add(m_options);
       m_options.setMnemonic(KeyEvent.VK_O);
       m_options.add(m_color);
          m_color.setMnemonic(KeyEvent.VK_C);
          m_color.add(mi_bcolor);
          m_color.add(mi_ncolor);
          m_color.add(mi_lcolor);
          m_color.add(mi_tcolor);
       m_options.add(mi_radius);
       m_options.addSeparator();
       m_options.add(mi_original);
       m_options.add(mi_custom);
       m_options.addSeparator();
       m_options.add(mi_save);
      mb_top.add(m_view);
       m_view.setMnemonic(KeyEvent.VK_V);
       m_view.add(mi_clear);
       m_view.addSeparator();
       m_view.add(mi_weight);
       m_view.add(mi_direction);
     mb_top.add(m_help);
       m_help.setMnemonic(KeyEvent.VK_H);
       m_help.add(mi_about);

      //add the listeners
      mi_new_1.addActionListener(this);
      mi_new_2.addActionListener(this);
      mi_sava.addActionListener(this);
      mi_new_d.addActionListener(this);
      mi_new_u.addActionListener(this);
      m_view.addMenuListener(this); //this menu must be generated dynmically
      mi_clear.addActionListener(this);
      mi_exit.addActionListener(this);
      mi_bcolor.addActionListener(this);
      mi_tcolor.addActionListener(this);
      mi_ncolor.addActionListener(this);
      mi_lcolor.addActionListener(this);
      mi_original.addActionListener(this);
      mi_radius.addActionListener(this);
      mi_save.addActionListener(this);
      mi_weight.addActionListener(this);
      mi_direction.addActionListener(this);
      mi_about.addActionListener(this);
      mi_custom.addActionListener(this);
      mi_load.addActionListener(this);
      mi_sav.addActionListener(this);

      //adds a window adapter
      this.addWindowListener(new WindowAdapter() {
           private boolean exit = false;

           public void windowClosing(WindowEvent e)
           {
	     if(graph.getModified()) //the graph has been modified
             {
               int save = JOptionPane.showConfirmDialog(new JFrame(),"Your graph has been modified. Would you like to save your work?","Save Work",JOptionPane.YES_NO_CANCEL_OPTION);
               if(save == 0) //they want to save,
               {
                  if(graph.save() == true) //save was successful
                  { exit = true; } //otherwise it will still be false and user will fall back to program
               }
               else if(save == 1)//they don't want to save, exit
               { exit = true; }
             }
             else //graph wasn't modified
             { exit = true; }

             if(exit)
             {
              GraphGUI.delInstance();         //decrement the window counter
              ( (JFrame) e.getSource() ).dispose(); //get rid of the window
              if(GraphGUI.getInstance() == 0) //check to see if we were the last one
               { System.exit(0); }        //if we were, close window
             }
           }
      });

      //put all the stuff onto the frame
      getContentPane().add(mb_top, BorderLayout.NORTH); //adds menubar to the top
      getContentPane().add(new StatusBar(graph), BorderLayout.SOUTH); //and status bar
      getContentPane().add(jsp_graph, BorderLayout.CENTER); //adds graph to center

      //set default close operation to do nothing. We'll let the WindowAdapter handle this
      setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

      //sets default title
      if(graph.getCurrentFile() != null)
        { setTitle("GraphMapper "+graph.getCurrentFile().getName()); }

      //increment our instance to let us know how many of us are running
      GraphGUI.addInstance();
   }

   /**
     *increment our instance count.<BR>
     *this is used to keep track of how many windows are open so we know when to close
     */
   private static void addInstance()
     { instances++; }
   /**
     *decrements our instance count.<BR>
     *this is used to keep track of how many windows are open so we know when to close
     */
   private static void delInstance()
     { instances--; }
   /**
     *gets our current instance count.<BR>
     *@return current number of windows open. Program should close if this is zero.
     */
   private static int getInstance()
     { return instances; }

   //our primary action listener for the menu
   public void actionPerformed(ActionEvent e)
   {
      if(e.getSource() == mi_sava)
      {
        graph.saveAs();
      }
      if(e.getSource() == mi_about)
      {
         JOptionPane.showMessageDialog(new JFrame(), GraphMapper.ABOUT, "GraphMapper", JOptionPane.PLAIN_MESSAGE);
      }
      else if(e.getSource() == mi_new_d)//new>directed graph
      {
         GraphGUI a = new GraphGUI(true);
         a.setSize(640,480);
         a.show();
      }
      else if(e.getSource() == mi_new_u)//new undirected
      {
         GraphGUI a = new GraphGUI(false);
         a.setSize(640,480);
         a.show();
      }
      else if(e.getSource() == mi_new_1)//new 
      {
         GraphGUI a = new GraphGUI(false,true);
         a.setSize(640,480);
         a.show();
      }
      else if(e.getSource() == mi_new_2)//new 
      {
         GraphGUI a = new GraphGUI(true,false);
         a.setSize(640,480);
         a.show();
      }
      else if(e.getSource() == mi_save)
      {
          INI.save(graph,new File("./.gmapper.ini") );
      }
      else if(e.getSource() == mi_clear)
      {
          graph.clearSearch();
      }
      else if(e.getSource() == mi_weight) //display status of weights
        {
          if(graph.getWeights() == true) //if on turn off
           { graph.setWeights(false); }
          else if(graph.getWeights() == false) //if off turn on
           { graph.setWeights(true); }
        }
      else if(e.getSource() == mi_direction)
        {
          if(graph.getArrows() == true) //if on turn off
           { graph.setArrows(false); }
          else if(graph.getArrows() == false) //if off turn on
           { graph.setArrows(true); }
        }
      else if(e.getSource() == mi_exit) //exit
        {
           //closes the window using it's window adapter
           this.dispatchEvent(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
        }
      //user wishes to change the default background color
      else if(e.getSource() == mi_bcolor)
        {
          boolean btest = true;
           while (btest) //if they didn't click cancel
           {
             Color bcolor = JColorChooser.showDialog(new JFrame(),"Background Color",graph.getBackground()); //give them a dialogue with the existing color selected
              if(bcolor != null && (bcolor.equals(graph.getTextColor()) || bcolor.equals(graph.getNodeColor()) || bcolor.equals(graph.getLineColor() )) ) //make sure colors don't clash
               {  JOptionPane.showMessageDialog(new JFrame(),"Background color can not be the same as node, line or text color","Error",JOptionPane.ERROR_MESSAGE);}
              else if(bcolor != null)
               {  graph.setBackground(bcolor);
                  btest=false;
               }
              else
               { btest = false; } //break us out of loop if they hit cancle
           }
        }
      //change default node color
      else if(e.getSource() == mi_ncolor)
      {
           boolean ntest = true;
           while (ntest) //if they didn't click cancel
           {
             Color ncolor = JColorChooser.showDialog(new JFrame(),"Node Color",graph.getNodeColor()); //give them a dialogue with the existing color selected
              if(ncolor != null && (ncolor.equals(graph.getTextColor()) || ncolor.equals(graph.getBackground())) ) //make sure colors don't clash
               {  JOptionPane.showMessageDialog(new JFrame(),"Node color can not be the same as background or text color","Error",JOptionPane.ERROR_MESSAGE);}
              else if(ncolor != null)
               {  graph.setNodeColor(ncolor);
                  ntest=false;
               }
              else
               { ntest = false; } //break us out of loop if they hit cancle
           }
      }
      //change default text color
      else if(e.getSource() == mi_tcolor)
      {
        boolean ttest = true;
        while(ttest)
        {
           Color tcolor = JColorChooser.showDialog(new JFrame(),"Text Color",graph.getTextColor()); //give them a dialogue with the existing color selected
           if (tcolor != null) //if they didn't click cancel
           {
              if(tcolor.equals(graph.getBackground()) || tcolor.equals(graph.getNodeColor()) ) //make sure colors don't clash
               {  JOptionPane.showMessageDialog(new JFrame(),"Text color can not be the same as background or node color","Error",JOptionPane.ERROR_MESSAGE);}
              else
               {
                 graph.setTextColor(tcolor);
                 ttest = false;
               }
           }
           else
           { ttest = false; }
        }
      }
      //change default line color
      else if(e.getSource() == mi_lcolor)
      {
        boolean ltest = true;
        while(ltest)
        {
           Color lcolor = JColorChooser.showDialog(new JFrame(),"Line Color",graph.getLineColor()); //give them a dialogue with the existing color selected
           if (lcolor != null) //if they didn't click cancel
           {
              if(lcolor.equals(graph.getBackground()) ) //make sure colors don't clash
               {  JOptionPane.showMessageDialog(new JFrame(),"Line color can not be the same as background color","Error",JOptionPane.ERROR_MESSAGE);}
              else
               {
                 graph.setLineColor(lcolor);
                 ltest = false;
               }
           }
           else
           { ltest = false; }
        }
      }
        //change radius
      else if(e.getSource() ==mi_radius)
      {
             //NOTE -- taken and modified from the popup menu class
          boolean valid = false;
          Integer i = new Integer(graph.getRadius());
          while(valid == false)
          {
            Object str_rad = JOptionPane.showInputDialog(new JFrame(), "Radius of Node", "Radius",JOptionPane.QUESTION_MESSAGE,null,null,i);
            if(str_rad != null) //if user didn't click cancle
            {
              try
              {
                int x = Integer.parseInt((String) str_rad); //check to see if they entered an integer
                if(x > 0) //check to see if int was positive
                 {
                    graph.setRadius(x); //set the new radius
                    valid = true; //break out of loop
                 }
                else
                 {  throw new NumberFormatException("Invalid radius"); } //rethrow exception for invalid numbers
              }
              catch(NumberFormatException f)
              {
                 JOptionPane.showMessageDialog(new JFrame(), "Invalid Radius", "Error", JOptionPane.ERROR_MESSAGE);
              }
            }
            else
              {valid = true;} //user clicked cancle
          }//end while loop
      }//end radius change if statement
     
      else if(e.getSource() == mi_custom) //remove customized nodes
      {
         if(JOptionPane.showConfirmDialog(new JFrame(), "Are you sure you want to remove all custom colors from the nodes and reset their radii?","Reset Custom Nodes",JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE) == 0)
         {
            graph.resetAllNodes();
         }
      }
      
      else if(e.getSource() == mi_original) //restore defaults
      {
         if(JOptionPane.showConfirmDialog(new JFrame(), "Are you sure you want to restore the default colors and radii","Reset Default Options",JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE) == 0)
         {
            graph.resetDefaults();
         }
      }

      else if(e.getSource() == mi_sav) //user wishes to save graph
      {
        graph.save();
      }
      else if(e.getSource() == mi_load)
      {
        graph.load();
      }
   }//end actionPerformed function


//we use this to dynmically generate information for the view menu
//on the fly since it's hard to keep trace of when files are loaded and saved
public void menuSelected(MenuEvent e)
{
      if(e.getSource() == m_view) //someone clicked the view menu
        {
          //what should be enable?
          if(graph.getSearchVector() == null) //sets the "Clear Search" greyed out state
           { mi_clear.setEnabled(false); }
          else
           { mi_clear.setEnabled(true); }

          if(graph.isWeighted()) //sets the enabled status of "show weights"
           { mi_weight.setEnabled(true); }
          else
           { mi_weight.setEnabled(false); }

          if(graph.isDirected()) //sets the enabled status of "show arrows"
           { mi_direction.setEnabled(true); }
          else
           { mi_direction.setEnabled(false); }

          //what are the current setting?
          if(graph.getArrows() == true && graph.isDirected() ) //are the arrows being drawn
            {mi_direction.setSelected(true);}
          else
            {mi_direction.setSelected(false);}
          if(graph.getWeights() == true && graph.isWeighted() ) //does the user want weights drawn
            {mi_weight.setSelected(true);}
          else
            {mi_weight.setSelected(false);}

        }
}
//extra functions from the interface I probably won't use
public void menuDeselected(MenuEvent e)
{
}

public void menuCanceled(MenuEvent e)
{
}


 
  //start thread for scroll refresh 
  //I later made this thread maintain the Graph Title too
  public void run()
  {
     while(0==0)
     {
       if(! graph.isScrollFresh()) //see if the scrollbar needs to be refreshed
       {
          jsp_graph.setViewportView(graph); //this is the only function I could find that actually reset the scroll bar. It looks like
          graph.setScrollFresh(true);       //it shouldn't work, but it does. Afterwards, set the scrollfresh to true.
       }


       if(graph.getCurrentFile() != null && !(title.equals(graph.getCurrentFile().getName())) )
       {
         title = graph.getCurrentFile().getName();
         this.setTitle( "GraphMapper "+title );
       }
          try
          { Thread.sleep(10); } //sleep the thread...don't put this in the if statement, it won't be pretty ~:-D
          catch(InterruptedException e) 
          { System.err.println("Scroll Panel Thread in trouble"); //I hope we never get to this point
          }

     }
  }        
  public void start() //gets our thread going
  {
     Thread t = new Thread(this);
     t.start();
  }

}//end class


/**
  *this class is used to read and write to the .gmgraph.ini file stored in the default running directory.
  */
class INI
{
   /**
     *integer values which must be saved.
     */
   private static int bcolor, ncolor, lcolor, tcolor, radius;
   /**
     *boolean values which must be saved.
     */
   private static boolean weight, arrows;   
  /**
    *saves the current settings to an ini file.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>must be passed a graph from which default values will be retrieved.</LI>
    *  <LI>must be passed a file to write default items to.</LI>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>INI file will be written or error given.</LI>
    *</UL><BR>
    */
   public static void save(JGraph g, File f)
   {
      bcolor = g.getBackground().getRGB();
      ncolor = g.getNodeColor().getRGB();
      lcolor = g.getLineColor().getRGB();
      tcolor = g.getTextColor().getRGB();
      weight = g.getWeights();
      arrows = g.getArrows();
      radius = g.getRadius();
     try
     {
        BufferedWriter write  = new BufferedWriter(new FileWriter(f));
        write.write("background="+bcolor);
        write.newLine();
        write.write("node="+ncolor);
        write.newLine();
	write.write("line="+lcolor);
	write.newLine();
        write.write("text="+tcolor);
        write.newLine();
        write.write("weight="+weight);
        write.newLine();
        write.write("arrows="+arrows);
        write.newLine();
        write.write("radius="+radius);
        write.flush();
        write.close();
        JOptionPane.showMessageDialog(new JFrame(),"Default Settings Saved",".gmapper.ini",JOptionPane.INFORMATION_MESSAGE);
     }
     catch(Exception io)
     {
        JOptionPane.showMessageDialog(new JFrame(),"Error Writing Setup file","Input/Ouput Error",JOptionPane.ERROR_MESSAGE);
     }
   }
  /**
    *loads the current settings from an ini file.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>must be passed a graph from which default values will be added.</LI>
    *  <LI>must be passed a file to read default items from.</LI>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>INI file will be read and JGraph set or error given.</LI>
    *</UL><BR>
    */
   public static void load(JGraph g, File f)
   {
      try
      {
        BufferedReader read = new BufferedReader(new FileReader(f));
        String line = read.readLine();
        while(line != null)
        {
          if(line.startsWith("background=") )
          {
             g.setBackground( new Color ( Integer.parseInt( line.substring(11) ) ) );
          }
          else if(line.startsWith("node="))
          {
             g.setNodeColor( new Color ( Integer.parseInt( line.substring(5) ) ) );
          }
          else if(line.startsWith("line="))
          {
             g.setLineColor( new Color ( Integer.parseInt( line.substring(5) ) ) );
          }
          else if(line.startsWith("text="))
          {
             g.setTextColor( new Color ( Integer.parseInt( line.substring(5) ) ) );
          }
          else if(line.startsWith("weight="))
          {
             g.setWeights( line.substring(7,11).equals(new String("true")) ); //I tried using Boolean.getBoolean()...it didn't work :(
          }
          else if(line.startsWith("arrows="))
          {
             g.setArrows( line.substring(7,11).equals(new String("true"))  );
          }
          else if(line.startsWith("radius="))
          {
             g.setRadius( Integer.parseInt( line.substring(7) ) );
          }
          line = read.readLine();
        }//end while loop
       read.close();
      }//end try block
      catch(Exception io) //all exceptions...everything from a bad file to being unable to parse and int
      {

        System.err.println(io);
        JOptionPane.showMessageDialog(new JFrame(),"Error Reading .gmapper.ini","Input/Ouput Error",JOptionPane.ERROR_MESSAGE);
      }
   }
}
