/**
  *Interface for Graph ADT.
  *This interface provides a way to insert, delete and retrieve
  *entries from a graph data structer.
  */
import java.util.Vector;

public interface GraphInterface 
{

   //two constructors
   //default constructor makes a directed graph
   //constructor that takes a boolean (true for directed)


  /**
    *clears the graph.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The graph will be emptied
    *</UL>
    */   
   public void makeEmpty();

  /**
    *determines if the graph is empty.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>A boolean will be returned stating the condition of the graph.
    *</UL>
    */
   public boolean isEmpty();

  /**
    *number of vertices in graph.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The number of vertices will be returned
    *</UL>
    */
   public int numVertices();

  /**
    *number of edges in a graph.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The number of edges will be returned
    *</UL>
    */
   public int numEdges();
 
  /**
    *adds a vertex to the graph.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>A <code>GraphNode</code>must be passed to it
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The vertex will be added to the graph.</LI>
    *  <LI>A <code>GraphException</code> will be thrown if a duplicate is added
    *</UL><BR>
    */
   public void addVertex(GraphNode myItem) throws GraphException;

  /**
    *adds an edge to the graph.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>The <code>SearchKeys</code>must be valid
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The Edge is added to the graph</LI>
    *  <LI>The weight is set to 1</LI>
    *</UL><BR>
    */
   public void addEdge(Comparable searchKey1, Comparable searchKey2) throws GraphException;

  /**
    *adds an edge to the graph.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>The <code>SearchKeys</code> must be valid and the weight specified
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>An edge will be added to the graph with the specified weight
    *</UL><BR>
    */
   public void addEdge(Comparable searchKey1, Comparable searchKey2, double weight) throws GraphException;

  /**
    *retrieves the weight of an edge.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>The <code>SearchKeys</code> must be valid
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The weight of the edge specified by the search keys will be returned
    *</UL><BR>
    */
   public double getWeight(Comparable searchKey1, Comparable searchKey2) throws GraphException;

  /**
    *removes an edge.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>The <code>SearchKeys</code> must be valid
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The egde will be removed from the graph</LI>
    *  <LI>If no edge existed, nothing will be changed</LI>
    *</UL><BR>
    */
   public void removeEdge(Comparable searchKey1, Comparable searchKey2) throws GraphException;

  /**
    *removes a Vertex.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>The <code>SearchKey</code> must be valid
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The Vertex will be removed
    *</UL><BR>
    */
   public GraphNode removeVertex(Comparable searchKey) throws GraphException;

  /**
    *retrieves an object from the graph.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>The <code>SearchKey</code> must be valid
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The <code>GraphNode</code> will be returned
    *</UL><BR>
    */
   public GraphNode getVertex(Comparable searchKey) throws GraphException;
  /**
    *retrieves an object from the graph.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>The <code>Index</code> must be valid
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The <code>GraphNode</code> will be returned
    *</UL><BR>
    */
   public GraphNode getVertex(int index) throws GraphException;

  /**
    *calculates the depth first search of the graph.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>The <code>searchKey</code> will be the starting position and it must be valid.
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>A <code>Vector</code> will return verticies visited in the order that they were visited during the search.
    *</UL><BR>
    */
   public Vector dfs(Comparable searchKey) throws GraphException;

  /**
    *calculates the breadth first search of the graph.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>The <code>searchKey</code> will be the starting position and it must be valid.
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>A <code>Vector</code> will return verticies visited in the order that they were visited during the search.
    *</UL><BR>
    */
   public Vector bfs(Comparable searchKey) throws GraphException;

  /**
    *calculates the shortest path between two vertices.<BR>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>The <code>searchKeys</code> for the start and end points must be valid. 
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>A <code>Vector</code> will return verticies resulting in the shortest path.
    *</UL><BR>
    */
   public Vector shortestPath(Comparable searchKey1, Comparable searchKey2) throws GraphException;

}
