/**
  *This class is used to create the popup menu when the user right clicks on the node.
  */

import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JPopupMenu;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JColorChooser;


public class GraphPopup extends JPopupMenu implements ActionListener
 {

    //create all our menus
    private JMenu edges = new JMenu("Edges");
      private JMenuItem edge = new JMenuItem("Create",KeyEvent.VK_C);
      private JMenuItem uedge= new JMenuItem("Remove",KeyEvent.VK_R);
    private JMenuItem del  = new JMenuItem("Remove Node",KeyEvent.VK_N);
    private JMenu search = new JMenu("Search");
      private JMenuItem path= new JMenuItem("Shortest Path", KeyEvent.VK_P);
      private JMenuItem bfs = new JMenuItem("Breadth First Search", KeyEvent.VK_B);
      private JMenuItem dfs = new JMenuItem("Depth First Search", KeyEvent.VK_D);
    private JMenu n_color = new JMenu("Node Color");
      private JRadioButtonMenuItem n_chose   = new JRadioButtonMenuItem("custom...");
      private JRadioButtonMenuItem n_default = new JRadioButtonMenuItem("default");
    private JMenu s_color = new JMenu("Text Color");
      private JRadioButtonMenuItem s_chose   = new JRadioButtonMenuItem("custom...");
      private JRadioButtonMenuItem s_default = new JRadioButtonMenuItem("default");
    private JMenu radius  = new JMenu("Radius");
      private JRadioButtonMenuItem r_chose   = new JRadioButtonMenuItem("custom...");
      private JRadioButtonMenuItem r_default = new JRadioButtonMenuItem("default");

    //component that refered us
    private JGraph c;

    //node we're dealing with
    GraphNode node;


   /**
     *creates a popup menu for a node.<br>
     *<B>Preconditions</B><BR>
     *<UL>
     *  <LI>Must be passed a JGraph to preform popup menu functions on</LI>
     *  <LI>Must be passed the x and y cord of the top left corner of the popup</LI>
     *  <LI>Must be passed a graphnode to perfrom popup menu functions on</LI>
     *</UL><BR>
     *<B>Postconditions</B><BR>
     *<UL>
     *  <LI>The user will be able to manipulate the Grpah and Node using this Popup
     *</UL>
     */
   public GraphPopup(JGraph c, int x, int y, GraphNode node)
   {

    //set refering component
    this.c = c;
    //set refering node
    this.node = node;

    //put them together
      edges.add(edge);
      edges.add(uedge);
      search.add(bfs);
      search.add(dfs);
      search.add(path);
      n_color.add(n_chose);
      n_color.add(n_default);
      s_color.add(s_chose);
      s_color.add(s_default);
      radius.add(r_chose);
      radius.add(r_default);

    //add keystrokes
    edges.setMnemonic(KeyEvent.VK_E);
    search.setMnemonic(KeyEvent.VK_S);
    n_color.setMnemonic(KeyEvent.VK_C);
    s_color.setMnemonic(KeyEvent.VK_T);
    radius.setMnemonic(KeyEvent.VK_R);
    //put them in popup
    this.add(edges);
    this.addSeparator();
    this.add(search);
    this.add(n_color);
    this.add(s_color);
    this.add(radius);
    this.addSeparator();
    this.add(del);

    //add action listeners
    uedge.addActionListener(this);
    del.addActionListener(this);
    edge.addActionListener(this);
    n_color.addActionListener(this);
    n_chose.addActionListener(this);
    s_color.addActionListener(this);
    s_chose.addActionListener(this);
    r_chose.addActionListener(this);
    r_default.addActionListener(this);
    s_default.addActionListener(this);
    n_default.addActionListener(this);
    bfs.addActionListener(this);
    dfs.addActionListener(this);
    path.addActionListener(this);

    //check nodes for enable status of radio buttons
    if(node.getRadius() <= 0)
     { r_default.setSelected(true); }
    else
     { r_chose.setSelected(true); }

    if(node.getColor() == null)
     { n_default.setSelected(true); }
    else
     { n_chose.setSelected(true); }

    if(node.getTextColor() == null)
     { s_default.setSelected(true); }
    else
     { s_chose.setSelected(true); }

     //display this popup with "c" as the invoker
     //and x,y the location of the top left corner of popup
     this.show(c,x,y);
   }
    public void actionPerformed(ActionEvent e)
    {
       if(e.getSource()==uedge) //remove an edge
       {
         c.setClickHot(true);
         c.setClickHotNode(node);
         c.setHotReason(c.UNLINK);
         c.setStatus("Select Destination Node");
       }
       if(e.getSource()==del) //delete a node
       {
         c.removeNode(node);
       }
       if(e.getSource()==edge) //drawn an edge
       {
         c.setClickHot(true);
         c.setClickHotNode(node);
         c.setHotReason(c.LINK);
         c.setStatus("Select Destination Node");
       }
       if(e.getSource()==path)
       {
         c.setClickHot(true);
         c.setClickHotNode(node);
         c.setHotReason(c.SHORT);
         c.setStatus("Select Destination Node");
       }
       if(e.getSource()==bfs) //Breth First Search
       {
          c.bfs(node); //send it back to the component since component has access to graph
       }
       if(e.getSource()==dfs) //dept first search
       {
          c.dfs(node); //send it back to the component since component has access to graph
       }
       else if(e.getSource()==n_default) //set node to default color
       {
          //defaults are easy
          node.setColor(null);         //set the node's color value to null
          c.setModified(true);
          c.repaint();                 //repaint the component
       }
       else if(e.getSource()==n_chose) //set node to custom color
       {
          Color temp = JColorChooser.showDialog(new JFrame(), "Node Color",node.getColor()); //show them color picker
          if(temp != null) //if they actually picked a color without hiting cancel
           {
              node.setColor(temp);
              c.setModified(true);
              c.repaint();
           }
       }
       else if(e.getSource()==s_default) //set text to default color
       {
          //defaults are easy
          node.setTextColor(null);         //set the node's color value to null
          c.setModified(true);
          c.repaint();                 //repaint the component
       }
       else if(e.getSource()==s_chose) //set text to custom color
       {
          Color temp = JColorChooser.showDialog(new JFrame(), "Text Color",node.getColor()); //show them color picker
          if(temp != null) //if they actually picked a color without hiting cancel
           {
              node.setTextColor(temp);
              c.setModified(true);
              c.repaint();
           }
           //else //reset radio button
           //{  s_chose.setSelected(false); }
       }
       else if(e.getSource()==r_default) //set default radius
       {
          //defaults are easy
          node.setRadius(-1);         //set the node's radius value to null
          c.setModified(true);
          c.repaint();                 //repaint the component
       }
       else if(e.getSource()==r_chose) //set a custom radius
       {
          boolean valid = false;
          String  initial;
          if(node.getRadius() > 0) //if the node isn't using a default radius
           { initial = new Integer(node.getRadius()).toString(); } //set initial to it
          else //set the initial to a blank line
           { initial = "";}
          while(valid == false)
          {
            Object str_rad = JOptionPane.showInputDialog(new JFrame(), "Radius of Node", "Radius",JOptionPane.QUESTION_MESSAGE,null,null,initial);
            if(str_rad != null) //if user didn't click cancle
            {
              try
              {
                int x = Integer.parseInt((String) str_rad); //check to see if they entered an integer
                if(x > 0) //check to see if int was positive
                 {
                    node.setRadius(x); //set the new radius
                    c.setModified(true);
                    c.repaint();  //repaint with new radius
                    valid = true; //break out of loop
                 }
                else
                 {  throw new NumberFormatException("Invalid radius"); } //rethrow exception for invalid numbers
              }
              catch(NumberFormatException f)
              {
                 JOptionPane.showMessageDialog(new JFrame(), "Invalid Radius", "Error", JOptionPane.ERROR_MESSAGE);
              }
            }
            else
              {valid = true;} //user clicked cancle
          }//end if statement
       }//end elseif statement
    }//end while loops
  }//end actionPerformed function



