/**
  *A component that will handle graph data.
  *This component is desigend to handle user input
  *and construct a standard, directional, weighted and non-selflooping graph.
  */
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionListener;
import java.awt.Dimension;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JPopupMenu;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JColorChooser;
import javax.swing.SwingUtilities;
import javax.swing.JFileChooser;
import java.util.Vector;
import java.util.Enumeration;
import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import javax.swing.filechooser.FileFilter;
import java.io.IOException;
import java.io.Serializable;
import java.io.FileNotFoundException;
//sure, I could just import javax.swing.*, but... :-)


  //by extending a Component vs. a Canvar or JCompoent or heaven forbit a JContainer or JPanel
  //I've avoided several uncescarry layers. There are 4 layers from JPanel to java.lang.Object
  //can we say "overhead?"
public class JGraph extends Component implements MouseMotionListener
{
   /**
     *Holds the data structer for storing graph information.
     */
   private Graph graph;

   /**
     *Default background color.
     */
   private Color background;
   /**
     *Default node color.
     */
   private Color ncolor;
   /**
     *Default text color.
     */
   private Color text;
  /**
    *Default line color.
    */
   private Color lcolor;
   /**
     *Default radius.
     */
   private int radius;
   /**
     *reference to self needed for innerclass.
     */
   private JGraph component; 

   /**
     *is something waiting for a hot click.
     */
   private boolean clickhot;
   /**
     *the node that's active for a special click operation.
     */
   private GraphNode clickhotnode;
   /**
     *the current x position of my mouse.
     */
   protected int mx;
   /**
     *the current y position of my mouse.
     */
   protected int my;
   /**
     *the string representing the status bar.
     */
   protected String status;
   
   /**
     *to set the hot reason as a link between two nodes.
     */
   public static final int LINK  = 0001;
   /**
     *to se the hot reason as to calculate the shortest path.
     */
   public static final int SHORT = 0010;
   /**
     *to remove an edge.
     */
   public static final int UNLINK = 1001;
   /**
     *contains the current hotreason
     */
   private int hotreason;

   /**
     *indicates if the scroll bars need to be refreshed.
     */
   private boolean scroll_fresh;
   /**
     *the perfered width (for use with scrollbars).
     */
   private int px = 0;
   /**
     *the prefered height (for use with scrollbars).
     */
   private int py = 0;

   /**
     *the file chooser.
     */
   private JFileChooser saver = new JFileChooser(); //creates a new file choser

   /**
     *current search path vector being displayed.
     */
   private Vector svector;
   /**
     *current node being dragged.
     */
   private GraphNode dragnode;
   /**
     *should the weights be displayed.
     */
   private boolean weights;
   /**
     *should the arrows be displayed.
     */
   private boolean arrows;
   /**
     *type of search we're currenly using.
     */
   private String searchtype;
   /**
     *has the file been modified since last save?
     */
   private boolean modified;

   /**
     *current file being displayed.
     */
   private File currentfile;

   /**
     *default constructor to create a JGraph and set its default values.
     */
   public JGraph()
   {
      super();                  //initalize the regular JFrame
      background = Color.darkGray;  //default background
      ncolor     = Color.gray;  //default node color
      text       = Color.white;  //default text color
      lcolor     = Color.green; //default line color
      radius     = 20;           //default radius
      clickhot   = false;        //initally nothing should be waiting for a hot click
      graph = new Graph(true);  //create a directed Graph to store data in
      addMouseListener(ma);      //adds the inner-class mouse adapter we create at the bottom
      addMouseMotionListener(this); //lets you see a line when connecting nodes
      svector = null;           //initally our search vector is set to null
      weights = true;           //initally weights are turned on
      arrows = true;            //initally arrows are turned on
      modified = false;         //by default new graphs are unmodified

      //this is needed for a function inside the inner-class MouseAdapter extention later so it
      //can pass "this" to something else (it can't use "this" becasue it's in an inner class with no reference
      //to this.
      component = this;

      //setup our filechooser
      //add a filter
      saver.setFileFilter(new FileFilter() {
         public boolean accept(File f)
         {
            if( (! f.isDirectory()) && f.toString().toLowerCase().endsWith(".gnp") )
              { return true; }
            else if (f.isDirectory())
              { return true; }
            else
              { return false; }
         }
         public String getDescription()
         {
            return "Graph Node Map (*.gnp)";
         }
      });
      //adds  some other settings
      saver.setFileSelectionMode(JFileChooser.FILES_ONLY);
      saver.setMultiSelectionEnabled(false);
   }

   public JGraph(boolean b)
   {
     this();
     graph = new Graph(b); //overrides graph created by previus constructor
   }
   public JGraph(boolean weight, boolean direction)
   {
     this();
     graph = new Graph(weight, direction);
   }
   /**
     *clears the search path.
     */
   public void clearSearch()
   {
     svector = null;
     status = "";
     repaint();
   }
   /**
     *returns the current file.
     *@return the current file or null if the file has not been saved
     */
   public File getCurrentFile()
   { return currentfile; }
   /**
     *returns if the graph is directed.
     *@return true if graph is directed
     */
   public boolean isDirected()
   { return graph.DIRECTED;   }
   /**
     *returns if the graph is weighted.
     *@returns true if graph is weighted
     */
   public boolean isWeighted()
   { return graph.WEIGHTED; }
   /**
     *sets the status of the arrows.
     *@param b status of arrows
     */
   public void setArrows(boolean b)
   { arrows = b; repaint(); modified = true;}
   /**
    *returns the current type of search.
    *@return type of search
    */
   public String getSearchType()
    { return searchtype; }
   /**
     *gets the status of the arrows.
     *@return status of arrows
     */
   public boolean getArrows()
    { return arrows; }
   /**
     *gets the status of weights.
     *@return status of weights
     */
   public boolean getWeights()
    { return weights; }
   /**
     *sets the status of weights.
     *@param b status of weights.
     */
   public void setWeights(boolean b)
     { weights = b; repaint(); modified = true;}
   /**
     *sets the background for my component.
     *@param col Color awt object for the background
     */
   public void setBackground(Color col)
   {
      background = col;
      modified = true;
      repaint();
   }
   /**
     *gets search vector.
     *@return search vector
     */
   public Vector getSearchVector()
   { return svector; }
   /**
     *sets the default node color.
     *@param col Color awt object for the node.
     */
   public void setNodeColor(Color col)
   {
      ncolor = col;
      modified = true;
      repaint();
   }
   /**
     *sets the default text color.
     *@param col Color awt object for text.
     */
   public void setTextColor(Color col)
   {
      text = col;
      modified = true;
      repaint();
   }
   /**
     *sets the default line color.
     *@param col Color awt object for line.
     */
   public void setLineColor(Color col)
   {
      lcolor = col;
      modified = true;
      repaint();
   }
   /**
     *sets the default radius.
     *@param x size of default radius (must be greater than zero).
     */
   public void setRadius(int x)
   {
      if(x <= 0)
       { throw new NumberFormatException("You can't have a negative radius you stupid person"); }
      else
       {
         radius = x;
         modified = true;
         repaint();
       }
   }
   /**
     *returns the current default radius.
     *@return default radius
     */
   public int getRadius()
    {return radius; }
   /**
     *returns defalut background color.
     *@return default background color.
     */
   public Color getBackground()
    { return background; }
   /**
     *returns the defautl node color.
     *@return default node color
     */
   public Color getNodeColor()
    { return ncolor; }
   /**
     *returns the default text color.
     *@return default text color
     */
   public Color getTextColor()
    { return text; }
   /**
     *returns the default line color.
     */
   public Color getLineColor()
    { return lcolor; }
   /**
     *resets the display to it's default colors and radii.
     */
   public void resetDefaults()
    {
      background = Color.darkGray;  //default background
      ncolor     = Color.gray;  //default node color
      text       = Color.white;  //default text color
      lcolor     = Color.green; //default line color
      radius     = 20;           //default radius
      modified = true;
      repaint();
    }
   /**
     *removes all customization from nodes.
     */
   public void resetAllNodes()
    {
       for(int x=0;x<graph.numVertices();x++)
       {
          GraphNode temp = graph.getVertex(x);
          temp.setRadius(0);
          temp.setColor(null);
          temp.setTextColor(null);
          modified = true;
          repaint();
       }
    }
   /**
     *removes a single node.
     *@param GraphNode the <code>GraphNode</code> to be removed
     */
   public void removeNode(GraphNode g)
   {
      graph.removeVertex(g.getKey());
      modified = true;
      repaint();
   }
   /**
     *bring up a dialog for the user to save GrapMap even if a file is all ready defined. 
     */
   public boolean saveAs()
   {
       File temp; //to swap out values
       temp = currentfile; //hold file or null
       currentfile = null;
       boolean b = save();
       if(! b) //file didn't save correctly
        {
          currentfile = temp; //reset it to the old value
        }
     return b;
   }
   /**
     *brings up a dialog for the user to save GraphMap or saves the file currently open.
     */
   public boolean save()
   {
      File save;
      int check;
      if(currentfile == null)
      {
         check = saver.showSaveDialog(new JFrame()); //pops up a save dialogue
         save = saver.getSelectedFile();
      }
      else
        {
          save = currentfile;
          check = JFileChooser.APPROVE_OPTION;
        }

      if(check == JFileChooser.APPROVE_OPTION) //the user clicked "save"
      {
        try{
         FileOutputStream outFile = new FileOutputStream( save ); //new output stream
         ObjectOutputStream outObject = new ObjectOutputStream(outFile); //wrap file stream in object stream
             SaveStruct s = new SaveStruct(); //create a new structer to save my values
             synchronized (s) //makes sure no other thread access our struct
             {
               s.background=this.background;
               s.ncolor = this.ncolor;
               s.lcolor = this.lcolor;
               s.tcolor = this.text;
               s.graph = this.graph;
               s.radius = this.radius;
               s.arrows = this.arrows;
               s.weights = this.weights;
               outObject.writeObject(s); //write savestruct to the stream
             }//send interaction with struct
            outObject.flush(); //push everything down the stream/empty queue
            outFile.close(); //resync with disk
            modified = false; //file on disk is now upto date with object in memory
            currentfile = save; //set current file
            return true; //file saved successfully
         }//end try
        catch(IOException crypt) //the standard, most useless and most cryptic output which can mean 1 of 100 things :-D
         {
            JOptionPane.showMessageDialog(new JFrame(),"Error writing to disk","Error",JOptionPane.ERROR_MESSAGE);
            return false;
         }
      }
      else
      { return false; /** user hit cancle **/ }
   }
   /**
     *brings up dialog for user to load GraphMap.
     *@return File - the File object the user loaded or null if user cancled
     */
   public File load()
   {
      int check = saver.showOpenDialog(new JFrame()); //give the user a dialog
      if(check == JFileChooser.APPROVE_OPTION) //if they selected a file to load
      {
      try{
         FileInputStream inFile = new FileInputStream( saver.getSelectedFile() );
         ObjectInputStream inObject = new ObjectInputStream(inFile);
         Object l = inObject.readObject();
          synchronized(l) //make sure no one else access struct while we use it
          {
            this.background = ( (SaveStruct) l).background;
            this.ncolor = ( (SaveStruct) l).ncolor;
            this.text = ( (SaveStruct) l).tcolor;
            this.lcolor = ( (SaveStruct) l).lcolor;
            this.graph = ( (SaveStruct) l).graph;
            this.radius = ( (SaveStruct) l).radius;
            this.arrows = ( (SaveStruct) l).arrows;
            this.weights= ( (SaveStruct) l).weights;
            inFile.close();
            modified = false; //file is upto date with object in memory`
            svector = null;
          }
          currentfile = saver.getSelectedFile();
          this.repaint(); //no need for repaint to be synchronized
          scroll_fresh = false; //tell thread to redo scrollbars
         }
         catch(Exception crypt) //this thing throws FIVE EXCEPTIONS! I'll just catch them all :-)
         {
             JOptionPane.showMessageDialog(new JFrame(),"Error loading file","Error",JOptionPane.ERROR_MESSAGE);
         }
     }
     return saver.getSelectedFile(); //user hit cancle this will be null, else it's the file loaded
   }
   /**
     *loads a file from the argument passed to it.
     *@param f File to load
     */
   public File load(File f)
   {
      try{
         FileInputStream inFile = new FileInputStream( f );
         ObjectInputStream inObject = new ObjectInputStream(inFile);
         Object l = inObject.readObject();
         synchronized(l) //make sure no one else tries to modify our struct while we're reading it
         {
            this.background = ( (SaveStruct) l).background;
            this.ncolor = ( (SaveStruct) l).ncolor;
            this.text = ( (SaveStruct) l).tcolor;
            this.lcolor = ( (SaveStruct) l).lcolor;
            this.graph = ( (SaveStruct) l).graph;
            this.radius = ( (SaveStruct) l).radius;
            this.arrows = ( (SaveStruct) l).arrows;
            this.weights= ( (SaveStruct) l).weights;
            inFile.close();
            modified = false;
            svector = null;
           }
          this.repaint(); //no need for repaint to be synchronized
          scroll_fresh = false; //tell thread to redo scrollbars
          currentfile = f;
         }
         catch(Exception crypt) //this thing throws FIVE EXCEPTIONS! I'll just catch them all :-)
         {
             JOptionPane.showMessageDialog(new JFrame(),"Error loading file","Error",JOptionPane.ERROR_MESSAGE);
             return null; //return a null if error occured with loading file
         }
      return f;
   }//end file load method
   /**
     *calls the Depth First Search and updates the display.
     *@param n Node to start search with
     */
   public void dfs(GraphNode n)
   {
      svector = graph.dfs(n.getKey());
      searchtype = "Depth First Search";
      repaint();
   }
   /**
     *calls the Bearth First Search and updates the display.
     *@param n Node to start search with
     */
   public void bfs(GraphNode n)
   {
      svector = graph.bfs(n.getKey());
      searchtype = "Breadth First Search";
      repaint();
   }
   /**
     *sets the click to <i>hot</i> meaning the user is using it for something other than default.
     */
   public void setClickHot(boolean h)
   { clickhot = h; }
   /**
     *sets the hot node.
     */
   public void setClickHotNode(GraphNode g)
   { clickhotnode = g; }
   /**
     *checks to see if the scroll bar might need to be refreshed.
     *@return boolean indicating the validty of the scrollbars.
     */
   public boolean isScrollFresh()
   { return scroll_fresh; }
   /**
     *sets the status of the scroll bar.
     *@param b validity of the scrollbar
     */
   public void setScrollFresh(boolean b)
   { scroll_fresh = b; }
   /**
     *sets the status bar text.
     *@param string to display in status bar.
     */
   public void setStatus(String s)
   { status = s; }

   /**
     *paints the component.
     *Do not call directly. Instead use repaint()
     */
   public void paint(Graphics g)
   {

      //functions for drawing backgrounds
      g.setColor(background);                   //set background color
      g.fillRect(0,0,getWidth(),getHeight());    //paint a rectange the size of the component

      //loop for drawing verticies
      double[][] dtemp = graph.edges; //the edge matrix
      //i //rows (source)
      //j //columns (destination)
      for(int i = 0; i < dtemp.length; i++) //go through each row
      {
          for(int j = 0; j < dtemp[i].length; j++) //go through each column of each row
          {
             if( dtemp[i][j] != Double.POSITIVE_INFINITY) //if it's not an infinity, drawn a line
             {
                //drawn a line from numberical value of i in vector to numberical value of j in vector
                int sx = ( (GraphNode) graph.items.elementAt(i) ).getX(); //source X
                int sy = ( (GraphNode) graph.items.elementAt(i) ).getY(); //source Y
                int dx = ( (GraphNode) graph.items.elementAt(j) ).getX(); //destination X
                int dy = ( (GraphNode) graph.items.elementAt(j) ).getY(); //destination Y

                int midx = (sx + dx) / 2;
                int midy = (sy + dy) / 2;

                //drawn line
                g.setColor(lcolor);
                g.drawLine(sx,sy,dx,dy);
		double a, b, c;
                int ax, ay, bx, by;
		// The following code was written my Marc. It handels the arrow drawing
                //using a complicated set of tring fuctions. I take no credit for this code. I barely
                //understand it and aside from cleaning it up and fixing
                //some special cases (verticle and horizontal lines), I haven't touched it.
                //Thanks Marc, you're awesome!!!
                //--oh I did draw the String, but Marc did all the trig stuff and drew the lines :-)

		if(sx < dx && sy > dy) //dest in first quadren
		{
			a = dx - sx; b = sy - dy;
			c = Math.sqrt(a * a + b * b);
                        ax = midx - (int) (25 * Math.cos(Math.acos(a/c)+Math.PI/6));
                        ay = midy + (int) (25 * Math.sin(Math.asin(b/c) + Math.PI/6));
                        bx = midx - (int) (25 * Math.cos(Math.acos(a/c)-Math.PI/6));
                        by = midy + (int) (25 * Math.sin(Math.asin(b/c) - Math.PI/6));
                          if(graph.WEIGHTED && weights && graph.DIRECTED)
                           { g.drawString(new Double(dtemp[i][j]).toString(),ax, ay+15); }
                        if(arrows & graph.DIRECTED ){
			g.drawLine(ax, ay, midx, midy);
			g.drawLine(bx, by, midx, midy);}
		}
		else if(sx <= dx && sy <= dy) //dest in fourth quad
		{
			a = dx - sx; b = dy - sy;
			c = Math.sqrt(a * a + b * b);
                        ax = midx - (int) (25 * Math.cos(Math.acos(a/c)-Math.PI/6));
                        ay = midy - (int) (25 * Math.sin(Math.asin(b/c) - Math.PI/6));
                        bx = midx - (int) (25 * Math.cos(Math.acos(a/c)+Math.PI/6));
                        by = midy - (int) (25 * Math.sin(Math.asin(b/c) + Math.PI/6));
                          if(graph.WEIGHTED && weights && graph.DIRECTED)
                           { g.drawString(new Double(dtemp[i][j]).toString(),bx, by-5); }
                        if(arrows && graph.DIRECTED) {
			g.drawLine(ax, ay, midx, midy);
			g.drawLine(bx, by, midx, midy);}
		}
		else if(sx > dx && sy <= dy) //dest in third quad
		{
			a = dx - sx; b = sy - dy;
			c = Math.sqrt(a * a + b * b);
                        ax = midx - (int) (25 * Math.cos(Math.acos(a/c)-Math.PI/6));
                        ay = midy + (int) (25 * Math.sin(Math.asin(b/c) - Math.PI/6));
                        bx = midx - (int) (25 * Math.cos(Math.acos(a/c)+Math.PI/6));
                        by = midy + (int) (25 * Math.sin(Math.asin(b/c) + Math.PI/6));
                          if(graph.WEIGHTED && weights && graph.DIRECTED)
                          { g.drawString(new Double(dtemp[i][j]).toString(),bx,by+15); }
                          else if(graph.WEIGHTED && weights) //needed for undirected weighted graphs
                           { g.drawString(new Double(dtemp[i][j]).toString(),midx-5,midy-10);}
                        if(arrows && graph.DIRECTED) {
			g.drawLine(ax,ay, midx, midy);
			g.drawLine(bx, by, midx, midy);}
		}
		else if(sx >= dx && sy >= dy) //dest in second quad
		{
			a = dx - sx; b = sy - dy;
			c = Math.sqrt(a * a + b * b);
                        ax = midx - (int) (25 * Math.cos(Math.acos(a/c)+Math.PI/6));
                        ay = midy + (int) (25 * Math.sin(Math.asin(b/c) - Math.PI/6));
                        bx = midx - (int) (25 * Math.cos(Math.acos(a/c)-Math.PI/6));
                        by = midy + (int) (25 * Math.sin(Math.asin(b/c) + Math.PI/6));
                          if(graph.WEIGHTED && weights && graph.DIRECTED)
                            { g.drawString(new Double(dtemp[i][j]).toString(),bx,by+15); }
                          else if(graph.WEIGHTED && weights) //needed for undirected weighted graphs
                            { g.drawString(new Double(dtemp[i][j]).toString(),midx-5,midy-10);}
                        if(arrows &&  graph.DIRECTED) {
			g.drawLine(ax, ay, midx, midy);
			g.drawLine(bx, by, midx, midy);}
		}
          }//if "if line exists" statement
        }//end inner for loop
      }//end outer for loop

      if(svector != null && (searchtype.equals("Shortest Path") ))//if a search was called and the vector is active
      {
         //split the colors of the spectrum by the number of segments you have
         //keep in mind RGB values range from -1 to -16777216
         int cstep = 30;/*350 / svector.size();*/
         int c = 30; //move this up to the first segment
         for(int x=0; x<svector.size()-1;x++) //loop through all segments
         {
             g.setColor(Color.getHSBColor( (float)(c*.001) ,1,1) ); //set the color for the first segment
             int sx = ( (GraphNode) svector.elementAt(x)).getX(); //get X and Y for source
             int sy = ( (GraphNode) svector.elementAt(x)).getY();
             int dx = ( (GraphNode) svector.elementAt(x+1)).getX(); //get X and Y for destination
             int dy = ( (GraphNode) svector.elementAt(x+1)).getY();
             g.drawLine(sx,sy,dx,dy); //draw segment

             c = c + cstep; //move up color spectrium
         }
      }

      GraphNode gn; //temp node
      px = 0; //reset prefered size each time
      py = 0; //reset prefered size each time
      for(int x = 0; x < graph.items.size(); x++) //loop over all nodes to draw them
      {
        gn = (GraphNode) graph.items.elementAt(x);
        if(gn != dragnode) //make sure we're not dragging anything
         {gn.drawNode(g, ncolor, text, radius);} //give each node a graphic object to draw themselves to
          //determine perfered size for scrollbars
               if(gn.getX() + gn.getRadius() + radius >= px) //if the x cord, plus the graphnode's radius plus the prefered radius is > than px
                 { px = gn.getX() + gn.getRadius() + radius; }
               if(gn.getY() + gn.getRadius() + radius >= py) //if the y cord, plus the graphnode's radius plus the prefered radius is > than py
                 { py = gn.getY() + gn.getRadius() + radius; }

      }
          //lastly draw our dragged node on top of other nodes if it exists
         if(dragnode != null)
         {dragnode.drawNode(g, ncolor, text, radius);}

      //if click is hot, this will allow for the line to be drawn to the cursor if the curson isn't moving
      //rather than simply using the mouseMoved function to drawn the line
      if(clickhot)
      {
         g.setColor(lcolor);
         int nx = clickhotnode.getX(); //the nodes x and y
         int ny = clickhotnode.getY();
         g.drawLine(nx,ny,mx,my); //draw the line to where my mouse is
      }
   }

  //OVERRIDDER methods so the correct ScrollPane viewport can be determined

   public Dimension getMinimumSize()
   {
     return new Dimension(px,py);
   }
   public Dimension getPreferredSize()
   {
     return new Dimension(px,py);
   }
   public Dimension getMaximumSize()
   {
     return new Dimension(px,py);
   }

   /**
     *sets the modified flag.
     *Use this function of you modify some part of the object so the user doesn't forget to save their changes.
     */
   public void setModified(boolean b)
   { modified = b; }
   /**
     *returns if the file has been modified since last save.
     */
   public boolean getModified()
   { return modified; }

   /**
     *the resson the mouse click his hot (eg shortest path, connect nodes, etc)
     */
   public void setHotReason(int r)
   {
      hotreason = r;
   }

   /**
     *mouse adapter to handle events to this canvas.
     */
   private MouseAdapter ma = new MouseAdapter() {
       public void mousePressed(MouseEvent e)
       {
	   dragnode = getNode(e.getX(),e.getY());
       }
       public void mouseReleased(MouseEvent e)
       {
           dragnode = null;
           repaint(); //so that draged node falls back behind other node
       }
       public void mouseClicked(MouseEvent e)
       {
          if(SwingUtilities.isLeftMouseButton(e) ) //left click
          {
           if(! clickhot ) //if another function hasn't specifically asked for the left click
           {
              GraphNode g_temp = getNode(e.getX(),e.getY()); //get the node at that location
               if(g_temp == null) //if there no no node there
               {
                 String name = JOptionPane.showInputDialog(new JFrame(),"Please Type In A Name For Your New Node","New Node",JOptionPane.QUESTION_MESSAGE);
                 if(name != null && name.trim().compareTo("") != 0 ) //the used typed something in instead of hitting cancel or closing box
                  {                                                  //also checks for whitespace
                     try
                     {
                        graph.addVertex(  new GraphNode( name,e.getX(),e.getY() )   ); //try and add the object
                        modified = true;
                        repaint(); //repaint component after we add somehting to it
                     }
                     catch(GraphException l) //check for duplicates
                     {
                        JOptionPane.showMessageDialog(new JFrame(),"Duplicate Node","Error",JOptionPane.ERROR_MESSAGE);
                     }
                  }
               }
               else
               {
                 //existing node, bring up dialogue
               }
           }
           else //some other fuction has requested the left click for something
           {
             if(hotreason == LINK)
             {
             //draw an edge
                     GraphNode temp = getNode(e.getX(),e.getY()); //get the node the user clicked
                     if(temp == null)//no node at that location
                     {
                        JOptionPane.showMessageDialog(new JFrame(),"Please Select A Node","",JOptionPane.WARNING_MESSAGE);
                     }
                     else if (temp == clickhotnode) //self loop
                     {
                             //give them a message and let them selct another node
                             JOptionPane.showMessageDialog(new JFrame(),"Self Loops Not Allowed","",JOptionPane.WARNING_MESSAGE);
                     }
                     else if(graph.WEIGHTED)
                     {
                       String str_edge; //used to store user output
                       //check to see if weight all ready exists
                       double n = graph.getWeight(clickhotnode.getKey(),temp.getKey());
                       if(n == Double.POSITIVE_INFINITY) //no weight exists
                       {str_edge =  JOptionPane.showInputDialog(new JFrame(), "What is the weight of this edge?", "Weight",JOptionPane.QUESTION_MESSAGE);}
                       else //show them the existing value
                       {str_edge = (String) JOptionPane.showInputDialog(new JFrame(), "What is the weight of this edge?", "Weight",JOptionPane.QUESTION_MESSAGE,null,null,new Double(n));}
                         try
                         {
                            double weight =Double.parseDouble(str_edge);
                            if(weight <= 0) //if they gave use a bad weight, pop and error and let it reloop
                              {
                                 JOptionPane.showMessageDialog(new JFrame(),"Invalid Weight","Error",JOptionPane.ERROR_MESSAGE);
                              }
                            else
                              {
                                graph.addEdge(clickhotnode.getKey(),temp.getKey(),weight);
                                clickhot = false; //turn off click hot after edge is established
                                modified = true; //set modified flag
                                repaint(); //paint the new edge
                              }
                         }
                         catch(NumberFormatException koop)
                         {
                            JOptionPane.showMessageDialog(new JFrame(),"Invalid Weight","Error",JOptionPane.ERROR_MESSAGE);
                         }
                         catch(NullPointerException bloop)
                         {
                            // we get this exception if we click cancle when is asks to enter a weight.
                            // we can also use it as a way to cancel clickhot
                            clickhot = false;
                            clickhotnode = null;
                         }
                     }
                     else if(!graph.WEIGHTED) //undirected graph
                     {
                       if(graph.edges[graph.items.indexOf(clickhotnode)][graph.items.indexOf(temp)] != Double.POSITIVE_INFINITY)
                        {JOptionPane.showMessageDialog(new JFrame(),"Edge All Ready Exists","",JOptionPane.WARNING_MESSAGE);}
                       else
                        {
                         graph.addEdge(clickhotnode.getKey(),temp.getKey());
                         clickhot = false;
                         modified = true;
                         repaint();
                        }
                     }
                  status = ""; //reset the status bar before we exit loop
             }//end if hotreason == LINK
             else if (hotreason == SHORT)
             {
               GraphNode stemp = getNode(e.getX(),e.getY());
                 if(stemp == null)
                 {
                    JOptionPane.showMessageDialog(new JFrame(),"Please click on a node","",JOptionPane.WARNING_MESSAGE);
                 }
                 else if(stemp == clickhotnode)
                 {
                    JOptionPane.showMessageDialog(new JFrame(),"No self looping paths allowed","Self Loop",JOptionPane.WARNING_MESSAGE);
                 }
                 else
                 {
                    try
                    {
                       svector = graph.shortestPath(clickhotnode.getKey(),stemp.getKey());
                       if(graph.WEIGHTED)
                         { setStatus("Distance of Shortest Path: "+graph.getDistance()) ;}
                       else
                         { setStatus("Number of Edges for Shortest Path: "+ ((int)graph.getDistance()) ) ;}
                       searchtype = "Shortest Path";
                       clickhot = false;
                       clickhotnode = null;
                       repaint();
                    }
                    catch(GraphException a)
                    {
                       JOptionPane.showMessageDialog(new JFrame(),"No shortest path possible","No Path",JOptionPane.INFORMATION_MESSAGE);
                       clickhot = false;
                       clickhotnode = null;
                       repaint();
                       setStatus("");
                    }
                 }
             }//end shortestpath elseif
             else if(hotreason == UNLINK) //delete a node
             {
               GraphNode deltemp = getNode(e.getX(),e.getY());
               {
                 if(deltemp == null)
                 {
                    JOptionPane.showMessageDialog(new JFrame(),"Please click on a node","",JOptionPane.WARNING_MESSAGE);
                 }
                 else if(graph.WEIGHTED) //if it's a directed graph, we can use the getWeight() function
                 {
                   if(graph.getWeight(clickhotnode.getKey(),deltemp.getKey()) == Double.POSITIVE_INFINITY) //no link there to being with
                   { JOptionPane.showMessageDialog(new JFrame(),"No existing link to delete","",JOptionPane.WARNING_MESSAGE); }
                   else
                   {
                     graph.removeEdge(clickhotnode.getKey(),deltemp.getKey());
                     modified = true;
                     clickhot = false;
                     clickhotnode = null;
                     repaint();
                   }
                 }
                 else if(! graph.WEIGHTED)
                 {
                   if(graph.edges[graph.items.indexOf(clickhotnode)][graph.items.indexOf(deltemp)] == Double.POSITIVE_INFINITY)
                   {
                     JOptionPane.showMessageDialog(new JFrame(),"No existing link to delete","",JOptionPane.WARNING_MESSAGE);
                   }
                   else
                   {
                     graph.removeEdge(clickhotnode.getKey(),deltemp.getKey());
                     modified = true;
                     clickhot = false;
                     clickhotnode = null;
                     repaint();
                   }
                  setStatus(""); //reset status
                 }
               }//end valid while loop
             }//end hot-reason-unlink
           }//end clickhot=true else statement
          }//end left mouse button if
          else if (SwingUtilities.isRightMouseButton(e)) //right button
          {
             if(! clickhot) //if the clickhot isn't on
             {
                  GraphNode g_temp = getNode(e.getX(),e.getY()); //get the node at that location
                  if( g_temp != null ) //there's a node there - popup click
                  {
                     new GraphPopup(component, e.getX(), e.getY(), g_temp); //popup our window
                  }
                  else //no node there - popup click
                  {
                     new GraphRPop(component, e.getX(), e.getY());
                  }
             }
             else //if clickhots on
             {
                  //we're going to use the right click as a cancle button when we're joining nodes
                  //the user will realize this when the line connecting the mousepointer and node disapears
                  clickhot = false;
                  clickhotnode = null; //reset the reference to the hot node (cleanup)
                  setStatus(""); //resets status bar
                  repaint(); //gets rid of leftover line
             }
          }
       }
   }; //END OF MOUSEADAPTER  (Big isn't it?)

   //and now for something more complicated, MouseMotionListner
   //this is used for drawing a line while you're trying to figure out
   //which node you want to add an edge to.
   //Here are the fuctions from the interface
    public void  mouseMoved(MouseEvent e)
     {
      if(clickhot)
       {
         mx = e.getX();            //my X and Y are set to the public fields so the paint function
         my = e.getY();           //will know here my cursor is
         repaint(); //so user can see line
       }
       else
       {
          //this simply sets the X,Y fields of this object incase another function needs it later
          //it also pervents the issue of the initial node connecting line starting in the top
          //left corner becasue X and Y are uninitalized (set to 0)
          mx = e.getX();
          my = e.getY();
       }
     }
   public void mouseDragged(MouseEvent e)
    {
       //and of course...the nodes are dragable :)
       //drag node
       if(dragnode != null && e.getX() > 0 && e.getY() > 0) //if a node actually exists there
       {
          dragnode.setX(e.getX()); //move the node
          dragnode.setY(e.getY());
          repaint(); //reapaint so you can see the new location
          modified = true;
          scroll_fresh = false; //readjust scrollbars

          //wierd, while mousePressed is active, mouseMoved isn't run. Repeating
          //those two settings in mouseMoved so status bar updates
           mx = e.getX();
           my = e.getY();
       }
    }

   /**
     *gets the node at a certain X/Y location.
     *@param x X cordinaite
     *@param y Y Cordinaite
     */
   private GraphNode getNode(int x, int y)
   {
      Enumeration etemp = graph.items.elements(); //get an enumerator to iterate through this vector
      while(etemp.hasMoreElements()) // keep going until we run out of elements
      {
         GraphNode gn_temp = (GraphNode) etemp.nextElement(); //get the next element

         double distance = Math.sqrt( Math.pow(gn_temp.getX() - x , 2) + Math.pow(gn_temp.getY() - y, 2)  ); //distance formula

         if(gn_temp.getRadius() <= 0) //the node has no radius
           {
             if ( distance < radius) //so use the default and compare
              { return gn_temp; }
           }
         else if(distance < gn_temp.getRadius()) //is this click within the node's radius
         { return gn_temp; }

      }
     return null; //not found, return null
   }

}

/**
  *a struct used to hold data before it's saved to/loaded from disk.
  *This is NOT an object, it simply holds data. Be sure to sync reads/writes
  *from this if you are using threads.
  */
class SaveStruct implements Serializable
{
   public Graph graph;
   public Color background, ncolor, tcolor, lcolor;
   public int radius;
   public boolean arrows, weights;
}


