/**
  *creates a search panel for a JGraph and adds it when nescarry to the EAST of a JFrame's BorderLayout.
  */


import javax.swing.JPanel;
import javax.swing.JFrame;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import java.awt.BorderLayout;
import java.util.Vector;




public class SearchPanel extends JPanel implements Runnable, ActionListener
{

   /**
     *indicates the type of search we've preformed.
     */
   private JLabel title = new JLabel();

   /**
     *indicates the result of the search.
     */
   private JList searchlist = new JList();

   /**
     *the button the clears the search and removes this panel.
     */
   private JButton clear = new JButton("Clear Search");

   /**
     *the graph that we search.
     */
   private JGraph graph;

   /**
     *the frame we add our panel to.
     */
   private JFrame frame;

   /**
     *indicates if the panel is in the JFrame.
     */
   private boolean up;

   /**
     *scrollbar for our data.
     */
   private JScrollPane scroller = new JScrollPane(searchlist);

   /**
     *constructs a search panel.<BR>
     *<B>Preconditions</B><BR>
     *<UL>
     *  <LI>Must be passed a JGraph</LI>
     *  <LI>Must be passed a JFrame</LI>
     *</UL><BR>
     *<B>Postconditions</B><BR>
     *<UL>
     *  <LI>Will monitor JGraph for information concering searches.</LI>
     *  <LI>Will add itself to the JFrame at BorderLayout.EAST when a search is being preformed</LI>
     *</UL><BR>
     */
   public SearchPanel(JGraph j, JFrame e)
   {
       graph = j; //sets the graph we're monitoring
       frame = e; //sets the frame we're monitoring
       //The layout for this is simply three components
       //1. Title in the north
       //2. List in the middle
       //3. Clear Button at bottom
       setLayout(new BorderLayout() ); 
       add(title, BorderLayout.NORTH);
          scroller = new JScrollPane(searchlist);
       add(scroller, BorderLayout.CENTER);
       add(clear, BorderLayout.SOUTH);
       up = false; //the panel isn't displaying initially
       clear.addActionListener(this);
       start(); //starts monitoring
   }

   //implemented from the Runnable interface
   //monitors our JGraph and updates the JPanel and JGraph if nescarry
   public void run()
   {
      while(true)//infinate loop
      {

         Vector temp =  graph.getSearchVector();
         if(temp != null && up == false) //initalize it 
         {
            up = true; //so we don't keep adding it more times than we need to
            title.setText(graph.getSearchType());
            searchlist.setListData(temp); //fill our list with vector
            frame.getContentPane().add(this, BorderLayout.EAST); //add to frame
         }
         else if(temp == null && up == true)//it's gone, take it off frame
         {
            up = false; //to keep it from being added again
            frame.getContentPane().remove(this);
         }
         else if(temp != null) //keep it up to date
         {
            title.setText(graph.getSearchType());
            searchlist.setListData(temp); //fill our list with vector
         }

         try{ Thread.sleep(100); } //sleep for a while
         catch(InterruptedException e){System.err.println("Search panel thread Interrupted"); }
      }
   }
   public void start()
   {
      Thread t = new Thread(this);
      t.start();
   }
   //for updating the JGraph if the user wants to clear the search
   public void actionPerformed(ActionEvent e)
   {
     if(e.getSource()==clear)
       {graph.clearSearch();}
   }

}

