/**
  *a JPanel thread to handle the status bar updates.
  */

import javax.swing.JPanel;
import javax.swing.JLabel;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dimension;

public class StatusBar extends JPanel implements Runnable
{
   /**
     *the status bar label
     */
   private JLabel status = new JLabel("", JLabel.CENTER);
   /**
     *the xy panel for the right corner.
     */
   private JPanel xy = new JPanel() ;
   /**
     *the label to display the x cord.
     */
   private JLabel xco = new JLabel() {
        //override getPreferredSize to keep JLabel status from shifting
        public Dimension getPreferredSize()
        {
           return new Dimension(60,15);
        }
     };
   /**
     *the label to display the y cord.
     */
   private JLabel yco = new JLabel(){
        public Dimension getPreferredSize()
        {
           return new Dimension(60,15);
        }
     };

  /**
    *the JGraph to get values from.
    */
   private JGraph graph;

   /**
     *constructs a status bar for a JGraph.<BR>
     *<B>Preconditions</B><BR>
     *<UL>
     *   <LI>must be passed a graph to monitor.
     *</UL><BR>
     *<B>Postconditions</B><BR>
     *<UL>
     *   <LI>the status bar will be continually updated with the x/y cords of the mouse.</LI>
     *   <LI>status messages will be continually updated</LI>
     *</UL>
     */
   public StatusBar(JGraph graph)
   {
      this.graph = graph; //estable the memory location of JGraph object
      setLayout(new BorderLayout()); //gives us a new borderlayout
        add(status, BorderLayout.CENTER); //puts the status label in the center
      xy.setLayout(new GridLayout(1,2)); //sets layout for x-y panel
        xy.add(xco);
        xy.add(yco);
      add(xy, BorderLayout.EAST);//add the xy panel to the right corner
      this.start();
   }

   //these two methods are implementations of Runnable and are
   //used to construct and run a parallel process
   //javadocs will be copied from interface

   public void start()
   {
           Thread thread = new Thread(this);
           thread.start();
   }
   public void run()
   {
      while(1==1)
      {
         xco.setText("X: " + graph.mx+"   ");
         yco.setText("Y: " + graph.my);
         status.setText(graph.status);
         try
          {Thread.sleep(10);} //1/10th second refresh rate
         catch(InterruptedException e)
          {
            System.err.println("Status Thread Just Died");
            System.err.println(e);
          }
      }
   }
}
